class VideoSettings:
    def __init__(self, bit_rate: int, fps: int, height: int, width: int) -> None:
        self.bit_rate = bit_rate
        self.fps = fps
        self.height = height
        self.width = width
